@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2015-2020 GEM Foundation
+# Copyright (C) 2015-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -49,10 +49,10 @@ class ReportWriter(object):
         'avglosses_data_transfer': 'Estimated data transfer for the avglosses',
         'exposure_info': 'Exposure model',
         'slow_sources': 'Slowest sources',
-        'task:classical_split_filter:0': 'Fastest task',
-        'task:classical_split_filter:-1': 'Slowest task',
+        'task:start_classical:0': 'Fastest task',
+        'task:start_classical:-1': 'Slowest task',
         'task_info': 'Information about the tasks',
-        'times_by_source_class': 'Computation times by source typology',
+        'eff_ruptures': 'Computation times by source typology',
         'performance': 'Slowest operations',
     }
 
@@ -67,7 +67,7 @@ class ReportWriter(object):
         versions = sorted(dstore['/'].attrs.items())
         self.text += '\n\n' + views.rst_table(versions)
         self.text += '\n\nnum_sites = %d, num_levels = %d, num_rlzs = %s' % (
-            len(dstore['sitecol']), len(oq.imtls.array), num_rlzs)
+            len(dstore['sitecol']), oq.imtls.size, num_rlzs)
 
     def add(self, name, obj=None):
         """Add the view named `name` to the report text"""
@@ -98,13 +98,13 @@ class ReportWriter(object):
             self.add('exposure_info')
         if 'source_info' in ds:
             self.add('slow_sources')
-            self.add('times_by_source_class')
+            self.add('eff_ruptures')
         if 'task_info' in ds:
             self.add('task_info')
             tasks = set(ds['task_info']['taskname'])
-            if 'classical_split_filter' in tasks:
-                self.add('task:classical_split_filter:0')
-                self.add('task:classical_split_filter:-1')
+            if 'start_classical' in tasks:
+                self.add('task:start_classical:0')
+                self.add('task:start_classical:-1')
             self.add('job_info')
         if 'performance_data' in ds:
             self.add('performance')
