@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2018-2020 GEM Foundation
+# Copyright (C) 2018-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -25,8 +25,7 @@ import numpy
 import h5py
 import zlib
 
-from openquake.baselib.general import random_filter, AccumDict, cached_property
-from openquake.hazardlib.calc.filters import SourceFilter
+from openquake.baselib.general import AccumDict, cached_property
 from openquake.hazardlib.source.base import BaseSeismicSource
 from openquake.hazardlib.geo.geodetic import min_geodetic_distance
 from openquake.hazardlib.geo.surface.planar import PlanarSurface
@@ -66,9 +65,9 @@ def convert_UCERFSource(self, node):
         source_file,
         self.investigation_time,
         start_date,
-        float(node["minMag"]),
+        node["minMag"],
         npd=self.convert_npdist(node),
-        hdd=self.convert_hpdist(node),
+        hdd=self.convert_hddist(node),
         aspect=~node.ruptAspectRatio,
         upper_seismogenic_depth=~node.pointGeometry.upperSeismoDepth,
         lower_seismogenic_depth=~node.pointGeometry.lowerSeismoDepth,
@@ -80,53 +79,6 @@ def convert_UCERFSource(self, node):
 SourceConverter.convert_UCERFSource = convert_UCERFSource
 
 
-class ImperfectPlanarSurface(PlanarSurface):
-    """
-    The planar surface class sets a narrow tolerance for the rectangular plane
-    to be distorted in cartesian space. Ruptures with aspect ratios << 1.0,
-    and with a dip of less than 90 degrees, cannot be generated in a manner
-    that is consistent with the definitions - and thus cannot be instantiated.
-    This subclass modifies the original planar surface class such that the
-    tolerance checks are over-ridden. We find that distance errors with respect
-    to a simple fault surface with a mesh spacing of 0.001 km are only on the
-    order of < 0.15 % for Rrup (< 2 % for Rjb, < 3.0E-5 % for Rx)
-    """
-    IMPERFECT_RECTANGLE_TOLERANCE = numpy.inf
-
-
-class UcerfFilter(SourceFilter):
-    """
-    Filter for UCERF sources, both background and faults.
-    """
-    def filter(self, srcs):
-        for src in srcs:
-            if hasattr(src, 'start'):  # fault sources
-                src.src_filter = self  # hack: needed for .iter_ruptures
-                src.all_ridx = src.get_ridx()
-                ridx = set()
-                for arr in src.all_ridx:
-                    ridx.update(arr)
-                src.indices = self.get_indices(src, ridx, src.mags.max())
-                if len(src.indices):
-                    yield src
-            else:  # background sources
-                yield from super().filter([src])
-
-    def get_indices(self, src, ridx, mag):
-        """
-        :param src: an UCERF source
-        :param ridx: a set of rupture indices
-        :param mag: magnitude to use to compute the integration distance
-        :returns: array with the IDs of the sites close to the ruptures
-        """
-        centroids = src.get_centroids(ridx)
-        mindistance = min_geodetic_distance(
-            (centroids[:, 0], centroids[:, 1]), self.sitecol.xyz)
-        idist = self.integration_distance(DEFAULT_TRT, mag)
-        indices, = (mindistance <= idist).nonzero()
-        return indices
-
-
 class UCERFSource(BaseSeismicSource):
     """
     :param source_file:
@@ -200,19 +152,20 @@ class UCERFSource(BaseSeismicSource):
     def mags(self):
         # read from FM0_0/MEANFS/MEANMSR/Magnitude
         with h5py.File(self.source_file, "r") as hdf5:
-            return hdf5[self.idx_set["mag"]][self.start: self.stop]
+            arr = hdf5[self.ukey["mag"]][self.start: self.stop]
+        return arr
 
     @cached_property
     def rate(self):
         # read from FM0_0/MEANFS/MEANMSR/Rates/MeanRates
         with h5py.File(self.source_file, "r") as hdf5:
-            return hdf5[self.idx_set["rate"]][self.start: self.stop]
+            return hdf5[self.ukey["rate"]][self.start: self.stop]
 
     @cached_property
     def rake(self):
         # read from FM0_0/MEANFS/Rake
         with h5py.File(self.source_file, "r") as hdf5:
-            return hdf5[self.idx_set["rake"]][self.start:self.stop]
+            return hdf5[self.ukey["rake"]][self.start:self.stop]
 
     def wkt(self):
         return ''
@@ -223,20 +176,21 @@ class UCERFSource(BaseSeismicSource):
         """
         return self.num_ruptures
 
-    def new(self, grp_id, branch_id):
+    def new(self, et_id, branch_id):
         """
-        :param grp_id: ordinal of the source group
+        :param et_id: ordinal of the source group
         :param branch_name: name of the UCERF branch
         :param branch_id: string associated to the branch
         :returns: a new UCERFSource associated to the branch_id
         """
         new = copy.copy(self)
-        new.grp_id = grp_id
-        new.source_id = branch_id
-        new.idx_set = build_idx_set(branch_id, self.start_date)
+        new.et_id = et_id
+        new.source_id = branch_id  # i.e. FM3_1/ABM/Shaw09Mod/
+        # DsrUni_CharConst_M5Rate6.5_MMaxOff7.3_NoFix_SpatSeisU2
+        new.ukey = build_ukey(branch_id, self.start_date)
         with h5py.File(self.source_file, "r") as hdf5:
             new.start = 0
-            new.stop = len(hdf5[new.idx_set["mag"]])
+            new.stop = len(hdf5[new.ukey["mag"]])
         return new
 
     def get_min_max_mag(self):
@@ -251,38 +205,31 @@ class UCERFSource(BaseSeismicSource):
         """
         return PoissonTOM(self.inv_time)
 
-    def get_ridx(self, iloc=None):
-        """List of rupture indices for the given iloc"""
-        with h5py.File(self.source_file, "r") as hdf5:
-            if iloc is None:
-                iloc = slice(self.start, self.stop)
-            return hdf5[self.idx_set["geol"] + "/RuptureIndex"][iloc]
+    def get_sections(self):
+        """
+        :returns: array of list of section indices
+        """
+        with h5py.File(self.source_file, 'r') as hdf5:
+            dset = hdf5[self.ukey["geol"] + "/RuptureIndex"]
+            return dset[self.start:self.stop]
 
-    def get_centroids(self, ridx):
+    def get_planes(self):
         """
-        :returns: array of centroids for the given rupture index
+        :returns: dictionary of planes, one per section
         """
-        centroids = []
-        with h5py.File(self.source_file, "r") as hdf5:
-            for idx in ridx:
-                trace = "{:s}/{:s}".format(self.idx_set["sec"], str(idx))
-                centroids.append(hdf5[trace + "/Centroids"][()])
-        return numpy.concatenate(centroids)
-
-    def gen_trace_planes(self, ridx):
-        """
-        :yields: trace and rupture planes for the given rupture index
-        """
-        with h5py.File(self.source_file, "r") as hdf5:
-            for idx in ridx:
-                trace = "{:s}/{:s}".format(self.idx_set["sec"], str(idx))
-                plane = hdf5[trace + "/RupturePlanes"][:].astype("float64")
-                yield trace, plane
+        dic = {}
+        with h5py.File(self.source_file, 'r') as hdf5:
+            sections = sorted(map(int, hdf5[self.ukey["sec"]]))
+            for sec in sections:
+                key = "{:s}/{:d}/RupturePlanes".format(self.ukey["sec"], sec)
+                dic[sec] = hdf5[key][:]
+        return dic
 
     def get_bounding_box(self, maxdist):
         """
         :returns: min_lon, min_lat, max_lon, max_lat
         """
+        # this is the bounding box of the background, i.e. all of California!
         with h5py.File(self.source_file, 'r') as hdf5:
             locations = hdf5["Grid/Locations"][()]
         lons, lats = locations[:, 0], locations[:, 1]
@@ -297,7 +244,7 @@ class UCERFSource(BaseSeismicSource):
         step - this is done here rather than in the sampling of the ruptures
         themselves
         """
-        branch_key = self.idx_set["grid_key"]
+        branch_key = self.ukey["grid_key"]
         with h5py.File(self.source_file, 'r') as hdf5:
             bg_locations = hdf5["Grid/Locations"][()]
             if hasattr(self, 'src_filter'):
@@ -318,45 +265,26 @@ class UCERFSource(BaseSeismicSource):
             # get list of indices from array of booleans
             return numpy.where(ok)[0].tolist()
 
-    def get_ucerf_rupture(self, iloc):
+    def get_ucerf_rupture(self, ridx):
         """
-        :param iloc:
-            Location of the rupture plane in the hdf5 file
+        :param ridx: rupture index
         """
-        trt = self.tectonic_region_type
-        if hasattr(self, 'all_ridx'):  # already computed by the UcerfFilter
-            ridx = self.all_ridx[iloc - self.start]
-        else:
-            ridx = self.get_ridx(iloc)
-        mag = self.mags[iloc - self.start]
+        sections = self.sections[ridx]
+        mag = self.mags[ridx]
+        if mag < self.min_mag:
+            return
+
         surface_set = []
-        indices = self.src_filter.get_indices(self, ridx, mag)
-        if len(indices) == 0:
-            return None
-        for trace, plane in self.gen_trace_planes(ridx):
-            # build simple fault surface
-            for jloc in range(0, plane.shape[2]):
-                top_left = Point(
-                    plane[0, 0, jloc], plane[0, 1, jloc], plane[0, 2, jloc])
-                top_right = Point(
-                    plane[1, 0, jloc], plane[1, 1, jloc], plane[1, 2, jloc])
-                bottom_right = Point(
-                    plane[2, 0, jloc], plane[2, 1, jloc], plane[2, 2, jloc])
-                bottom_left = Point(
-                    plane[3, 0, jloc], plane[3, 1, jloc], plane[3, 2, jloc])
-                try:
-                    surface_set.append(
-                        ImperfectPlanarSurface.from_corner_points(
-                            top_left, top_right, bottom_right, bottom_left))
-                except ValueError as err:
-                    raise ValueError(err, trace, top_left, top_right,
-                                     bottom_right, bottom_left)
+        for sec in sections:
+            plane = self.planes[sec]
+            for p in range(plane.shape[2]):
+                surface_set.append(PlanarSurface.from_ucerf(plane[:, :, p]))
 
         rupture = ParametricProbabilisticRupture(
-            mag, self.rake[iloc - self.start], trt,
+            mag, self.rake[ridx], self.tectonic_region_type,
             surface_set[len(surface_set) // 2].get_middle_point(),
-            MultiSurface(surface_set), self.rate[iloc - self.start], self.tom)
-
+            MultiSurface(surface_set), self.rate[ridx], self.tom)
+        rupture.rup_id = self.start + ridx
         return rupture
 
     def iter_ruptures(self, **kwargs):
@@ -365,11 +293,11 @@ class UCERFSource(BaseSeismicSource):
         """
         for ridx in range(self.start, self.stop):
             if self.rate[ridx - self.start]:  # may have have zero rate
-                rup = self.get_ucerf_rupture(ridx)
+                rup = self.get_ucerf_rupture(ridx - self.start)
                 if rup:
                     yield rup
 
-    # called upfront, before classical_split_filter
+    # called upfront, before start_classical
     def __iter__(self):
         if self.stop - self.start <= self.ruptures_per_block:  # already split
             yield self
@@ -386,19 +314,13 @@ class UCERFSource(BaseSeismicSource):
     def __repr__(self):
         return '<%s %s>' % (self.__class__.__name__, self.source_id)
 
-    def get_background_sources(self, sample_factor=None):
+    def get_background_sources(self):
         """
         Turn the background model of a given branch into a set of point sources
-
-        :param sample_factor:
-            Used to reduce the sources if OQ_SAMPLE_SOURCES is set
         """
         background_sids = self.get_background_sids()
-        if sample_factor is not None:  # hack for use in the mosaic
-            background_sids = random_filter(
-                background_sids, sample_factor, seed=42)
         with h5py.File(self.source_file, "r") as hdf5:
-            grid_loc = "/".join(["Grid", self.idx_set["grid_key"]])
+            grid_loc = "/".join(["Grid", self.ukey["grid_key"]])
             # for instance Grid/FM0_0_MEANFS_MEANMSR_MeanRates
             mags = hdf5[grid_loc + "/Magnitude"][()]
             mmax = hdf5[grid_loc + "/MMax"][background_sids]
@@ -406,8 +328,8 @@ class UCERFSource(BaseSeismicSource):
             locations = hdf5["Grid/Locations"][background_sids, :]
             sources = []
             for i, bg_idx in enumerate(background_sids):
-                src_id = "_".join([self.idx_set["grid_key"], str(bg_idx)])
-                src_name = "|".join([self.idx_set["total_key"], str(bg_idx)])
+                src_id = "_".join([self.ukey["grid_key"], str(bg_idx)])
+                src_name = "|".join([self.ukey["total_key"], str(bg_idx)])
                 mag_idx = (self.min_mag <= mags) & (mags < mmax[i])
                 src_mags = mags[mag_idx]
                 src_mfd = EvenlyDiscretizedMFD(
@@ -423,13 +345,14 @@ class UCERFSource(BaseSeismicSource):
                 ps.checksum = zlib.adler32(pickle.dumps(vars(ps), protocol=4))
                 ps._wkt = ps.wkt()
                 ps.id = self.id
-                ps.grp_id = self.grp_id
+                ps.et_id = self.et_id
                 ps.num_ruptures = ps.count_ruptures()
+                ps.nsites = 1  # anything <> 0 goes
                 sources.append(ps)
         return sources
 
-    def get_one_rupture(self):
-        raise ValueError('Unsupported option')
+    def get_one_rupture(self, ses_seed):
+        raise NotImplementedError
 
     def generate_event_set(self, background_sids, eff_num_ses):
         """
@@ -446,15 +369,15 @@ class UCERFSource(BaseSeismicSource):
             # get ruptures from the indices
             ruptures = []
             rupture_occ = []
-            for iloc, n_occ in zip(indices, occurrences[indices]):
-                ucerf_rup = self.get_ucerf_rupture(iloc)
+            for ridx, n_occ in zip(indices, occurrences[indices]):
+                ucerf_rup = self.get_ucerf_rupture(ridx)
                 if ucerf_rup:
                     ruptures.append(ucerf_rup)
                     rupture_occ.append(n_occ)
 
             # sample background sources
             background_ruptures, background_n_occ = sample_background_model(
-                hdf5, self.idx_set["grid_key"], self.tom, eff_num_ses,
+                hdf5, self.ukey["grid_key"], self.tom, eff_num_ses,
                 self.serial, background_sids, self.min_mag, self.npd,
                 self.hdd, self.usd, self.lsd, self.msr, self.aspect,
                 self.tectonic_region_type)
@@ -533,26 +456,26 @@ def sample_background_model(
 # #################################################################### #
 
 
-def build_idx_set(branch_id, start_date):
+def build_ukey(branch_id, start_date):
     """
     Builds a dictionary of keys based on the branch code
     """
     code_set = branch_id.split("/")
     code_set.insert(3, "Rates")
-    idx_set = {
+    ukey = {
         "sec": "/".join([code_set[0], code_set[1], "Sections"]),
         "mag": "/".join([code_set[0], code_set[1], code_set[2], "Magnitude"])}
-    idx_set["rate"] = "/".join(code_set)
-    idx_set["rake"] = "/".join([code_set[0], code_set[1], "Rake"])
-    idx_set["msr"] = "-".join(code_set[:3])
-    idx_set["geol"] = code_set[0]
+    ukey["rate"] = "/".join(code_set)
+    ukey["rake"] = "/".join([code_set[0], code_set[1], "Rake"])
+    ukey["msr"] = "-".join(code_set[:3])
+    ukey["geol"] = code_set[0]
     if start_date:  # time-dependent source
-        idx_set["grid_key"] = "_".join(
+        ukey["grid_key"] = "_".join(
             branch_id.replace("/", "_").split("_")[:-1])
     else:  # time-independent source
-        idx_set["grid_key"] = branch_id.replace("/", "_")
-    idx_set["total_key"] = branch_id.replace("/", "|")
-    return idx_set
+        ukey["grid_key"] = branch_id.replace("/", "_")
+    ukey["total_key"] = branch_id.replace("/", "|")
+    return ukey
 
 
 def get_rupture_dimensions(mag, nodal_plane, msr, rupture_aspect_ratio,
