@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2015-2020 GEM Foundation
+# Copyright (C) 2015-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -15,28 +15,30 @@
 #
 # You should have received a copy of the GNU Affero General Public License
 # along with OpenQuake. If not, see <http://www.gnu.org/licenses/>.
+
+import io
 import ast
 import os.path
 import numbers
 import operator
 import functools
 import collections
+import logging
 import numpy
+import pandas
 
 from openquake.baselib.general import (
     humansize, countby, AccumDict, CallableDict,
-    get_array, group_array, fast_agg, fast_agg3)
+    get_array, group_array, fast_agg)
+from openquake.baselib.hdf5 import FLOAT, INT, get_shape_descr
 from openquake.baselib.performance import performance_view
 from openquake.baselib.python3compat import encode, decode
-from openquake.hazardlib import valid
 from openquake.hazardlib.gsim.base import ContextMaker
-from openquake.commonlib import util, calc
-from openquake.commonlib.writers import build_header, scientificformat
-from openquake.calculators import getters
+from openquake.commonlib import util
+from openquake.commonlib.writers import (
+    build_header, scientificformat, write_csv)
 from openquake.calculators.extract import extract
 
-FLOAT = (float, numpy.float32, numpy.float64)
-INT = (int, numpy.int32, numpy.uint32, numpy.int64, numpy.uint64)
 F32 = numpy.float32
 U32 = numpy.uint32
 
@@ -91,7 +93,7 @@ def form(value):
 
 def rst_table(data, header=None, fmt=None):
     """
-    Build a .rst table from a matrix.
+    Build a .rst table from a matrix or a DataFrame
     
     >>> tbl = [['a', 1], ['b', 2]]
     >>> print(rst_table(tbl, header=['Name', 'Value']))
@@ -102,6 +104,11 @@ def rst_table(data, header=None, fmt=None):
     b    2    
     ==== =====
     """
+    if isinstance(data, pandas.DataFrame):
+        if data.index.name:
+            data = data.reset_index()
+        header = header or list(data.columns)
+        data = data.to_numpy()
     if header is None and hasattr(data, '_fields'):
         header = data._fields
     try:
@@ -140,29 +147,18 @@ def rst_table(data, header=None, fmt=None):
     return '\n'.join(lines)
 
 
-@view.add('times_by_source_class')
-def view_times_by_source_class(token, dstore):
-    """
-    Returns the calculation times depending on the source typology
-    """
-    totals = fast_agg3(dstore['source_info']['code', 'calc_time'],
-                       'code', ['calc_time'])
-    return rst_table(totals)
-
-
 @view.add('slow_sources')
 def view_slow_sources(token, dstore, maxrows=20):
     """
     Returns the slowest sources
     """
-    info = dstore['source_info']['source_id', 'code', 'multiplicity',
+    info = dstore['source_info']['source_id', 'code',
                                  'calc_time', 'num_sites', 'eff_ruptures']
     info = info[info['eff_ruptures'] > 0]
     info.sort(order='calc_time')
     data = numpy.zeros(len(info), [(nam, object) for nam in info.dtype.names])
     for name in info.dtype.names:
         data[name] = info[name]
-    data['num_sites'] /= data['eff_ruptures']
     return rst_table(data[::-1][:maxrows])
 
 
@@ -171,7 +167,7 @@ def view_slow_ruptures(token, dstore, maxrows=25):
     """
     Show the slowest ruptures
     """
-    fields = ['code', 'n_occ', 'mag', 'grp_id']
+    fields = ['code', 'n_occ', 'mag', 'et_id']
     rups = dstore['ruptures'][()][fields]
     time = dstore['gmf_data/time_by_rup'][()]
     arr = util.compose_arrays(rups, time)
@@ -199,24 +195,24 @@ def view_contents(token, dstore):
 @view.add('full_lt')
 def view_full_lt(token, dstore):
     full_lt = dstore['full_lt']
-    if full_lt.num_samples == 0:
-        num_rlzs = full_lt.gsim_lt.get_num_paths()
-    header = ['smlt_path', 'weight', 'num_realizations']
+    try:
+        rlzs_by_gsim_list = full_lt.get_rlzs_by_gsim_list(dstore['et_ids'])
+    except KeyError:  # for scenario et_ids is missing
+        rlzs_by_gsim_list = [full_lt.get_rlzs_by_gsim(0)]
+    header = ['grp_id', 'gsim', 'rlzs']
     rows = []
-    for sm in full_lt.sm_rlzs:
-        if full_lt.num_samples:
-            num_rlzs = sm.samples
-        row = ('_'.join(sm.lt_path), sm.weight, num_rlzs)
-        rows.append(row)
+    for grp_id, rbg in enumerate(rlzs_by_gsim_list):
+        for gsim, rlzs in rbg.items():
+            rows.append([grp_id, repr(str(gsim)), str(list(rlzs))])
     return rst_table(rows, header)
 
 
 @view.add('eff_ruptures')
 def view_eff_ruptures(token, dstore):
-    header = ['num_ruptures', 'eff_ruptures']
-    info = dstore['source_info']['num_ruptures', 'eff_ruptures']
-    return rst_table([[info['num_ruptures'].sum(),
-                       info['eff_ruptures'].sum()]], header)
+    info = dstore.read_df('source_info', 'source_id')
+    df = info.groupby('code').sum()
+    del df['grp_id'], df['trti'], df['task_no']
+    return rst_table(df)
 
 
 @view.add('short_source_info')
@@ -305,7 +301,7 @@ def avglosses_data_transfer(token, dstore):
     oq = dstore['oqparam']
     N = len(dstore['assetcol'])
     R = dstore['full_lt'].get_num_rlzs()
-    L = len(dstore.get_attr('risk_model', 'loss_types'))
+    L = len(dstore.get_attr('crm', 'loss_types'))
     ct = oq.concurrent_tasks
     size_bytes = N * R * L * 8 * ct  # 8 byte floats
     return (
@@ -323,18 +319,21 @@ def view_totlosses(token, dstore):
     sanity check for the correctness of the implementation.
     """
     oq = dstore['oqparam']
-    tot_losses = dstore['losses_by_asset']['mean'].sum(axis=0)
-    return rst_table(tot_losses.view(oq.loss_dt()), fmt='%.6E')
+    tot_losses = dstore['avg_losses-rlzs'][()].sum(axis=0)
+    return rst_table(tot_losses.view(oq.loss_dt(F32)), fmt='%.6E')
 
 
-# for event based risk and ebrisk
-def portfolio_loss(dstore):
+def _portfolio_loss(dstore):
     R = dstore['full_lt'].get_num_rlzs()
-    array = dstore['losses_by_event'][()]
-    L, = array.dtype['loss'].shape  # loss has shape L
+    K = dstore['agg_loss_table'].attrs.get('K', 0)
+    df = dstore.read_df('agg_loss_table', 'agg_id', dict(agg_id=K))
+    eids = df.pop('event_id').to_numpy()
+    loss = df.to_numpy()
+    rlzs = dstore['events']['rlz_id'][eids]
+    L = loss.shape[1]
     data = numpy.zeros((R, L), F32)
-    for row in array:
-        data[row['rlzi']] += row['loss']
+    for row, rlz in zip(loss, rlzs):
+        data[rlz] += row
     return data
 
 
@@ -346,7 +345,7 @@ def view_portfolio_losses(token, dstore):
     """
     oq = dstore['oqparam']
     loss_dt = oq.loss_dt()
-    data = portfolio_loss(dstore).view(loss_dt)[:, 0]
+    data = _portfolio_loss(dstore).view(loss_dt)[:, 0]
     rlzids = [str(r) for r in range(len(data))]
     array = util.compose_arrays(numpy.array(rlzids), data, 'rlz_id')
     # this is very sensitive to rounding errors, so I am using a low precision
@@ -356,15 +355,49 @@ def view_portfolio_losses(token, dstore):
 @view.add('portfolio_loss')
 def view_portfolio_loss(token, dstore):
     """
-    The mean and stddev loss for the full portfolio for each loss type,
-    extracted from the event loss table, averaged over the realizations
+    The mean portfolio loss for each loss type,
+    extracted from the event loss table.
     """
-    data = portfolio_loss(dstore)  # shape (R, L)
-    loss_types = list(dstore['oqparam'].loss_dt().names)
-    header = ['portfolio_loss'] + loss_types
-    mean = ['mean'] + [row.mean() for row in data.T]
-    stddev = ['stddev'] + [row.std(ddof=1) for row in data.T]
-    return rst_table([mean, stddev], header)
+    oq = dstore['oqparam']
+    R = dstore['full_lt'].get_num_rlzs()
+    K = dstore['agg_loss_table'].attrs.get('K', 0)
+    df = dstore.read_df('agg_loss_table', 'agg_id', dict(agg_id=K))
+    weights = dstore['weights'][:]
+    rlzs = dstore['events']['rlz_id']
+    ws = weights[rlzs]
+    eids = df.pop('event_id').to_numpy()
+    arr = df.to_numpy()
+    E, L = arr.shape
+    avg = ws[eids] @ arr / ws.sum() * E / R
+    # err = [avg[li] * binning_error(arr[:, li], eids) for li in range(L)]
+    rows = [['avg'] + list(avg)]  # , ['err'] + err]
+    return(rst_table(rows, ['loss'] + oq.loss_names))
+
+
+@view.add('portfolio_damage_error')
+def portfolio_damage_error(token, dstore):
+    """
+    The damages and errors for the full portfolio, extracted from
+    the asset damage table.
+    """
+    return dstore.read_df('avg_portfolio_damage')
+
+
+@view.add('portfolio_damage')
+def view_portfolio_damage(token, dstore):
+    """
+    The mean full portfolio damage for each loss type,
+    extracted from the average damages
+    """
+    # dimensions assets, stat, loss_types, dmg_state
+    if 'damages-stats' in dstore:
+        attrs = get_shape_descr(dstore['damages-stats'].attrs['json'])
+        arr = dstore.sel('damages-stats', stat='mean').sum(axis=(0, 1))
+    else:
+        attrs = get_shape_descr(dstore['damages-rlzs'].attrs['json'])
+        arr = dstore.sel('damages-rlzs', rlz=0).sum(axis=(0, 1))
+    rows = [[lt] + list(row) for lt, row in zip(attrs['loss_type'], arr)]
+    return rst_table(rows, ['loss_type'] + list(attrs['dmg_state']))
 
 
 def sum_table(records):
@@ -435,11 +468,12 @@ def stats(name, array, *extras):
     Returns statistics from an array of numbers.
 
     :param name: a descriptive string
-    :returns: (name, mean, std, min, max, len)
+    :returns: (name, mean, rel_std, min, max, len)
     """
-    std = numpy.nan if len(array) == 1 else numpy.std(array, ddof=1)
-    return (name, numpy.mean(array), std,
-            numpy.min(array), numpy.max(array), len(array)) + extras
+    avg = numpy.mean(array)
+    std = 'nan' if len(array) == 1 else '%d%%' % (numpy.std(array) / avg * 100)
+    return (name, len(array), avg, std,
+            numpy.min(array), numpy.max(array)) + extras
 
 
 @view.add('num_units')
@@ -463,7 +497,7 @@ def view_assets_by_site(token, dstore):
     """
     taxonomies = dstore['assetcol/tagcol/taxonomy'][()]
     assets_by_site = dstore['assetcol'].assets_by_site()
-    data = ['taxonomy mean stddev min max num_sites num_assets'.split()]
+    data = ['taxonomy num_assets mean stddev min max num_sites'.split()]
     num_assets = AccumDict()
     for assets in assets_by_site:
         num_assets += {k: [len(v)] for k, v in group_array(
@@ -484,16 +518,16 @@ def view_required_params_per_trt(token, dstore):
     """
     full_lt = dstore['full_lt']
     tbl = []
-    for grp_id, trt in sorted(full_lt.trt_by_grp.items()):
+    for et_id, trt in enumerate(full_lt.trt_by_et):
         gsims = full_lt.gsim_lt.get_gsims(trt)
         maker = ContextMaker(trt, gsims)
         distances = sorted(maker.REQUIRES_DISTANCES)
         siteparams = sorted(maker.REQUIRES_SITES_PARAMETERS)
         ruptparams = sorted(maker.REQUIRES_RUPTURE_PARAMETERS)
-        tbl.append((grp_id, ' '.join(map(repr, map(repr, gsims))),
+        tbl.append((et_id, ' '.join(map(repr, map(repr, gsims))),
                     distances, siteparams, ruptparams))
     return rst_table(
-        tbl, header='grp_id gsims distances siteparams ruptparams'.split(),
+        tbl, header='et_id gsims distances siteparams ruptparams'.split(),
         fmt=scientificformat)
 
 
@@ -517,7 +551,7 @@ def view_task_info(token, dstore):
         data.sort(order='duration')
         return rst_table(data)
 
-    data = ['operation-duration mean stddev min max outputs'.split()]
+    data = ['operation-duration counts mean stddev min max'.split()]
     for task, arr in group_array(task_info[()], 'taskname').items():
         val = arr['duration']
         if len(val):
@@ -593,45 +627,17 @@ def view_task_ebrisk(token, dstore):
     return msg
 
 
-@view.add('hmap')
-def view_hmap(token, dstore):
+@view.add('global_hazard')
+def view_global_hazard(token, dstore):
     """
-    Display the highest 20 points of the mean hazard map. Called as
-    $ oq show hmap:0.1  # 10% PoE
+    Display the global hazard for the calculation. This is used for
+    debugging purposes when comparing the results of two
+    calculations.
     """
-    try:
-        poe = valid.probability(token.split(':')[1])
-    except IndexError:
-        poe = 0.1
-    mean = dict(extract(dstore, 'hcurves?kind=mean'))['mean']
-    oq = dstore['oqparam']
-    hmap = calc.make_hmap_array(mean, oq.imtls, [poe], len(mean))
-    dt = numpy.dtype([('sid', U32)] + [(imt, F32) for imt in oq.imtls])
-    array = numpy.zeros(len(hmap), dt)
-    for i, vals in enumerate(hmap):
-        array[i] = (i, ) + tuple(vals)
-    array.sort(order=list(oq.imtls)[0])
-    return rst_table(array[:20])
-
-
-@view.add('global_hcurves')
-def view_global_hcurves(token, dstore):
-    """
-    Display the global hazard curves for the calculation. They are
-    used for debugging purposes when comparing the results of two
-    calculations. They are the mean over the sites of the mean hazard
-    curves.
-    """
-    oq = dstore['oqparam']
-    nsites = len(dstore['sitecol'])
-    rlzs = dstore['full_lt'].get_realizations()
-    weights = [rlz.weight for rlz in rlzs]
-    mean = getters.PmapGetter(dstore, weights).get_mean()
-    array = calc.convert_to_array(mean, nsites, oq.imtls)
-    res = numpy.zeros(1, array.dtype)
-    for name in array.dtype.names:
-        res[name] = array[name].mean()
-    return rst_table(res)
+    imtls = dstore['oqparam'].imtls
+    arr = dstore.sel('hcurves-stats', stat='mean')  # shape N, S, M, L
+    res = arr.mean(axis=(0, 1, 3))  # shape M
+    return rst_table([res], imtls)
 
 
 @view.add('global_poes')
@@ -641,7 +647,10 @@ def view_global_poes(token, dstore):
     """
     tbl = []
     imtls = dstore['oqparam'].imtls
-    header = ['grp_id'] + [str(poe) for poe in imtls.array]
+    header = ['et_id']
+    for imt in imtls:
+        for poe in imtls[imt]:
+            header.append(str(poe))
     for grp in sorted(dstore['poes']):
         poes = dstore['poes/' + grp]
         nsites = len(poes)
@@ -672,25 +681,92 @@ def view_global_hmaps(token, dstore):
 @view.add('global_gmfs')
 def view_global_gmfs(token, dstore):
     """
-    Display GMFs averaged on everything for debugging purposes
+    Display GMFs on the first IMT averaged on everything for debugging purposes
     """
     imtls = dstore['oqparam'].imtls
-    row = dstore['gmf_data/data']['gmv'].mean(axis=0)
+    row = [dstore[f'gmf_data/gmv_{m}'][:].mean(axis=0)
+           for m in range(len(imtls))]
     return rst_table([row], header=imtls)
 
 
-@view.add('gmv_by_rup')
-def view_gmv_by_rup(token, dstore):
+@view.add('gmf')
+def view_gmf(token, dstore):
     """
-    Display a synthetic gmv per rupture serial for debugging purposes
+    Display a mean gmf for debugging purposes
     """
-    rup_id = dstore['events']['rup_id']
-    serial = dstore['ruptures']['serial']
-    data = dstore['gmf_data/data'][()]
-    gmv = fast_agg3(data, 'eid', ['gmv'])
-    gmv['eid'] = serial[rup_id[gmv['eid']]]
-    gm = fast_agg3(gmv, 'eid', ['gmv'])
-    return rst_table(gm, header=['serial', 'gmv'])
+    df = dstore.read_df('gmf_data', 'sid')
+    gmf = df.groupby(df.index).mean()
+    return str(gmf)
+
+
+def get_gmv0(dstore):
+    # returns dict gmf_error, extreme_ruptures
+    eids = dstore['gmf_data/eid'][:]
+    gmvs = dstore['gmf_data/gmv_0'][:]
+    sids = dstore['gmf_data/sid'][:]
+    df = pandas.DataFrame({'gmv_0': gmvs, 'sid': sids}, eids)
+    return df
+
+
+def binning_error(values, eids, nbins=10):
+    """
+    :param values: E values
+    :param eids: E integer event indices
+    :returns: std/mean for the sums of the values
+
+    Group the values in nbins depending on the eids and returns the
+    variability of the sums relative to the mean.
+    """
+    df = pandas.DataFrame({'val': values}, eids)
+    res = df.groupby(eids % nbins).val.sum()
+    return res.std() / res.mean()
+
+
+@view.add('gmf_error')
+def view_gmf_error(token, dstore):
+    """
+    Display a gmf relative error for seed dependency
+    """
+    return binning_error(
+        dstore['gmf_data/gmv_0'][:], dstore['gmf_data/eid'][:])
+
+
+class GmpeExtractor(object):
+    def __init__(self, dstore):
+        full_lt = dstore['full_lt']
+        self.trt_by_et = full_lt.trt_by_et
+        self.gsim_by_trt = full_lt.gsim_by_trt
+        self.rlzs = full_lt.get_realizations()
+
+    def extract(self, et_ids, rlz_ids):
+        out = []
+        for et_id, rlz_id in zip(et_ids, rlz_ids):
+            trt = self.trt_by_et[et_id]
+            out.append(self.gsim_by_trt(self.rlzs[rlz_id])[trt])
+        return out
+
+
+@view.add('extreme_gmvs')
+def view_extreme_gmvs(token, dstore):
+    """
+    Display table of extreme GMVs with fields (eid, gmv_0, sid, rlz. rup)
+    """
+    if ':' in token:
+        maxgmv = float(token.split(':')[1])
+    else:
+        maxgmv = 10  # 10g is default value defining extreme GMVs
+    imt0 = list(dstore['oqparam'].imtls)[0]
+    if imt0.startswith(('PGA', 'SA(')):
+        gmpe = GmpeExtractor(dstore)
+        df = get_gmv0(dstore)
+        extreme_df = df[df.gmv_0 > maxgmv].copy()
+        ev = dstore['events'][()][extreme_df.index]
+        extreme_df['rlz'] = ev['rlz_id']
+        extreme_df['rup'] = ev['rup_id']
+        et_ids = dstore['ruptures']['et_id'][extreme_df.rup]
+        extreme_df['gmpe'] = gmpe.extract(et_ids, ev['rlz_id'])
+        return extreme_df.sort_values('gmv_0').groupby('sid').head(1)
+    return 'Could not do anything for ' + imt0
 
 
 @view.add('mean_disagg')
@@ -699,60 +775,32 @@ def view_mean_disagg(token, dstore):
     Display mean quantities for the disaggregation. Useful for checking
     differences between two calculations.
     """
+    N, M, P, Z = dstore['hmap4'].shape
     tbl = []
-    for key, dset in sorted(dstore['disagg'].items()):
-        vals = [ds[()].mean() for k, ds in sorted(dset.items())]
-        tbl.append([key] + vals)
-    header = ['key'] + sorted(dset)
-    return rst_table(sorted(tbl), header=header)
-
-
-@view.add('elt')
-def view_elt(token, dstore):
-    """
-    Display the event loss table averaged by event
-    """
+    kd = {key: dset[:] for key, dset in sorted(dstore['disagg'].items())}
     oq = dstore['oqparam']
-    R = len(dstore['full_lt'].rlzs)
-    dic = group_array(dstore['losses_by_event'][()], 'rlzi')
-    header = oq.loss_dt().names
+    for s in range(N):
+        for m, imt in enumerate(oq.imtls):
+            for p in range(P):
+                row = ['%s-sid-%d-poe-%s' % (imt, s, p)]
+                for k, d in kd.items():
+                    row.append(d[s, m, p].mean())
+                tbl.append(row)
+    return rst_table(sorted(tbl), header=['key'] + list(kd))
+
+
+@view.add('disagg_times')
+def view_disagg_times(token, dstore):
+    """
+    Display slow tasks for disaggregation
+    """
+    data = dstore['disagg_task'][:]
+    info = dstore.read_df('task_info', 'taskname').loc[b'compute_disagg']
     tbl = []
-    for rlzi in range(R):
-        if rlzi in dic:
-            tbl.append(dic[rlzi]['loss'].mean(axis=0))
-        else:
-            tbl.append([0.] * len(header))
-    return rst_table(tbl, header)
-
-
-@view.add('pmap')
-def view_pmap(token, dstore):
-    """
-    Display the mean ProbabilityMap associated to a given source group name
-    """
-    grp = token.split(':')[1]  # called as pmap:grp
-    pmap = {}
-    rlzs = dstore['full_lt'].get_realizations()
-    weights = [rlz.weight for rlz in rlzs]
-    pgetter = getters.PmapGetter(dstore, weights)
-    pmap = pgetter.get_mean(grp)
-    return str(pmap)
-
-
-@view.add('act_ruptures_by_src')
-def view_act_ruptures_by_src(token, dstore):
-    """
-    Display the actual number of ruptures by source in event based calculations
-    """
-    data = dstore['ruptures'][('srcidx', 'rup_id')]
-    counts = sorted(countby(data, 'srcidx').items(),
-                    key=operator.itemgetter(1), reverse=True)
-    src_info = dstore['source_info'][('grp_id', 'source_id')]
-    table = [['src_id', 'grp_id', 'act_ruptures']]
-    for srcidx, act_ruptures in counts:
-        src = src_info[srcidx]
-        table.append([src['source_id'], src['grp_id'], act_ruptures])
-    return rst_table(table)
+    for duration, task_no in zip(info['duration'], info['task_no']):
+        tbl.append((duration, task_no) + tuple(data[task_no]))
+    header = ('duration', 'task_no') + data.dtype.names
+    return rst_table(sorted(tbl), header=header)
 
 
 @view.add('bad_ruptures')
@@ -797,8 +845,8 @@ def view_gmvs_to_hazard(token, dstore):
     assert rlz < dstore['full_lt'].get_num_rlzs()
     oq = dstore['oqparam']
     num_ses = oq.ses_per_logic_tree_path
-    data = dstore['gmf_data/data'][()]
-    data = data[(data['sid'] == sid) & (data['rlzi'] == rlz)]
+    data = dstore.read_df('gmf_data', 'sid').loc[sid]
+    data = data['rlzi'] == rlz
     tbl = []
     gmv = data['gmv']
     for imti, (imt, imls) in enumerate(oq.imtls.items()):
@@ -817,8 +865,8 @@ def view_gmvs(token, dstore):
     """
     sid = int(token.split(':')[1])  # called as view_gmvs:sid
     assert sid in dstore['sitecol'].sids
-    data = dstore['gmf_data/data'][()]
-    gmvs = data[data['sid'] == sid]['gmv']
+    data = dstore.read_df('gmf_data', 'sid')
+    gmvs = data.loc[sid]['gmv']
     return rst_table(gmvs)
 
 
@@ -835,6 +883,16 @@ def view_events_by_mag(token, dstore):
     return rst_table(counts.items(), ['mag', 'num_events'])
 
 
+@view.add('ebrups_by_mag')
+def view_ebrups_by_mag(token, dstore):
+    """
+    Show how many event based ruptures there are for each magnitude
+    """
+    mags = dstore['ruptures']['mag']
+    uniq, counts = numpy.unique(mags, return_counts=True)
+    return rst_table(zip(uniq, counts), ['mag', 'num_ruptures'])
+
+
 @view.add('maximum_intensity')
 def view_maximum_intensity(token, dstore):
     """
@@ -843,3 +901,52 @@ def view_maximum_intensity(token, dstore):
     effect = extract(dstore, 'effect')
     data = zip(dstore['full_lt'].trts, effect[-1, -1], effect[-1, 0])
     return rst_table(data, ['trt', 'intensity1', 'intensity2'])
+
+
+@view.add('extreme_sites')
+def view_extreme(token, dstore):
+    """
+    Show sites where the mean hazard map reaches maximum values
+    """
+    mean = dstore.sel('hmaps-stats', stat='mean')[:, 0, 0, -1]  # shape N1MP
+    site_ids, = numpy.where(mean == mean.max())
+    arr = dstore['sitecol'][site_ids]
+    sio = io.StringIO()
+    write_csv(sio, arr)
+    return sio.getvalue()
+
+
+@view.add('zero_losses')
+def view_zero_losses(token, dstore):
+    """
+    Sanity check on avg_losses and avg_gmf
+    """
+    R = len(dstore['weights'])
+    oq = dstore['oqparam']
+    avg_gmf = dstore['avg_gmf'][0]
+    asset_df = dstore.read_df('assetcol/array', 'site_id')
+    for col in asset_df.columns:
+        if not col.startswith('value-'):
+            del asset_df[col]
+    values_df = asset_df.groupby(asset_df.index).sum()
+    avglosses = dstore['avg_losses-rlzs'][:].sum(axis=1) / R  # shape (A, L)
+    dic = dict(site_id=dstore['assetcol']['site_id'])
+    for lti, lname in enumerate(oq.loss_names):
+        dic[lname] = avglosses[:, lti]
+    losses_df = pandas.DataFrame(dic).groupby('site_id').sum()
+    sids = losses_df.index.to_numpy()
+    avg_gmf = avg_gmf[sids]
+    nonzero_gmf = (avg_gmf > oq.min_iml).any(axis=1)
+    nonzero_losses = (losses_df > 0).to_numpy().any(axis=1)
+    bad, = numpy.where(nonzero_gmf != nonzero_losses)
+    # this happens in scenario_risk/case_shakemap and case_3
+    msg = 'Site #%d is suspicious:\navg_gmf=%s\navg_loss=%s\nvalues=%s'
+    for idx in bad:
+        sid = sids[idx]
+        logging.warning(msg, sid, dict(zip(oq.all_imts(), avg_gmf[sid])),
+                        _get(losses_df, sid), _get(values_df, sid))
+    return bad
+
+
+def _get(df, sid):
+    return df.loc[sid].to_dict()
