@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2017-2020 GEM Foundation
+# Copyright (C) 2017-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -20,7 +20,6 @@ import shutil
 import sqlite3
 import os.path
 import tempfile
-from openquake.baselib import sap
 from openquake.baselib.general import safeprint, zipfiles
 from openquake.server.dbserver import db
 
@@ -38,15 +37,14 @@ def smart_save(dbpath, archive, calc_id):
             conn.execute('DELETE FROM job WHERE status != "complete"')
             if calc_id:
                 conn.execute('DELETE FROM job WHERE id != %d' % calc_id)
-    except:
+    except Exception:
         safeprint('Please check the copy of the db in %s' % newdb)
         raise
     zipfiles([newdb], archive, 'a', safeprint)
     shutil.rmtree(tmpdir)
 
 
-@sap.script
-def dump(archive, calc_id=0, user=None):
+def main(archive, calc_id: int = 0, *, user=None):
     """
     Dump the openquake database and all the complete calculations into a zip
     file. In a multiuser installation must be run as administrator.
@@ -77,7 +75,6 @@ def dump(archive, calc_id=0, user=None):
               % (len(fnames), archive, dt))
 
 
-dump.arg('archive', 'path to the zip file where to dump the calculations')
-dump.arg('calc_id', 'calculation ID; if missing, dump all calculations',
-         type=int)
-dump.opt('user', 'if missing, dump all calculations')
+main.archive = 'path to the zip file where to dump the calculations'
+main.calc_id = 'calculation ID; if missing, dump all calculations'
+main.user = 'if missing, dump all calculations'
