@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2018-2020 GEM Foundation
+# Copyright (C) 2018-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -53,13 +53,16 @@ def zip_source_model(ssmLT, archive_zip='', log=logging.info):
     archive_zip = archive_zip or os.path.join(basedir, 'ssmLT.zip')
     if os.path.exists(archive_zip):
         sys.exit('%s exists already' % archive_zip)
-    oq = mock.Mock(inputs={'source_model_logic_tree': ssmLT})
+    smlt = logictree.SourceModelLogicTree(ssmLT)
+    files = list(smlt.hdf5_files) + smlt.info.smpaths
+    oq = mock.Mock(inputs={'source_model_logic_tree': ssmLT},
+                   random_seed=42, number_of_logic_tree_samples=0,
+                   sampling_method='early_weights')
     checksum = readinput.get_checksum32(oq)
     checkfile = os.path.join(os.path.dirname(ssmLT), 'CHECKSUM.txt')
     with open(checkfile, 'w') as f:
         f.write(str(checksum))
-    files = [os.path.abspath(ssmLT), os.path.abspath(checkfile)]
-    files.extend(logictree.collect_info(ssmLT).smpaths)
+    files.extend([os.path.abspath(ssmLT), os.path.abspath(checkfile)])
     general.zipfiles(files, archive_zip, log=log, cleanup=True)
     return archive_zip
 
@@ -94,7 +97,7 @@ def zip_job(job_ini, archive_zip='', risk_ini='', oq=None, log=logging.info):
     oq = oq or readinput.get_oqparam(job_ini, validate=False)
     if risk_ini:
         risk_ini = os.path.normpath(os.path.abspath(risk_ini))
-        risk_inputs = readinput.get_params([risk_ini])['inputs']
+        risk_inputs = readinput.get_params(risk_ini)['inputs']
         del risk_inputs['job_ini']
         oq.inputs.update(risk_inputs)
     files = readinput.get_input_files(oq)
