@@ -64,8 +64,8 @@ class CauzziEtAl2014RhypoGermany(CauzziEtAl2014):
     REQUIRES_DISTANCES = {"rhypo", "rrup"}
     REQUIRES_RUPTURE_PARAMETERS = {"rake", "mag", "width"}
 
-    def __init__(self, adjustment_factor=1.0):
-        super().__init__()
+    def __init__(self, adjustment_factor=1.0, **kwargs):
+        super().__init__(adjustment_factor=adjustment_factor, **kwargs)
         self.adjustment_factor = np.log(adjustment_factor)
 
     def _compute_mean(self, C, rup, dists, sites, imt):
@@ -106,8 +106,8 @@ class DerrasEtAl2014RhypoGermany(DerrasEtAl2014):
     REQUIRES_DISTANCES = {'rjb', 'rhypo'}
     REQUIRES_RUPTURE_PARAMETERS = {"rake", "mag", "hypo_depth", "width"}
 
-    def __init__(self, adjustment_factor=1.0):
-        super().__init__()
+    def __init__(self, adjustment_factor=1.0, **kwargs):
+        super().__init__(adjustment_factor=adjustment_factor, **kwargs)
         self.adjustment_factor = np.log(adjustment_factor)
 
     def get_mean_and_stddevs(self, sites, rup, dists, imt, stddev_types):
