@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2012-2020 GEM Foundation
+# Copyright (C) 2012-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -47,10 +47,7 @@ class ZhaoEtAl2006Asc(GMPE):
     #: Supported intensity measure types are spectral acceleration,
     #: and peak ground acceleration, see paragraph 'Development of Base Model'
     #: p. 901.
-    DEFINED_FOR_INTENSITY_MEASURE_TYPES = set([
-        PGA,
-        SA
-    ])
+    DEFINED_FOR_INTENSITY_MEASURE_TYPES = {PGA, SA}
 
     #: Supported intensity measure component is geometric mean
     #: of two horizontal components :
@@ -60,11 +57,10 @@ class ZhaoEtAl2006Asc(GMPE):
 
     #: Supported standard deviation types are inter-event, intra-event
     #: and total, see equation 3, p. 902.
-    DEFINED_FOR_STANDARD_DEVIATION_TYPES = set([
+    DEFINED_FOR_STANDARD_DEVIATION_TYPES = {
         const.StdDev.TOTAL,
         const.StdDev.INTER_EVENT,
-        const.StdDev.INTRA_EVENT
-    ])
+        const.StdDev.INTRA_EVENT}
 
     #: Required site parameters is Vs30.
     #: See table 2, p. 901.
@@ -78,6 +74,12 @@ class ZhaoEtAl2006Asc(GMPE):
     #: See paragraph 'Development of Base Model', p. 902.
     REQUIRES_DISTANCES = {'rrup'}
 
+    #: Reference conditions. See Table 2 at page 901. The hard rock conditions
+    #: is 1100 m/s. Here we force it to 800 to make it compatible with a 
+    #: generic site term
+    #  DEFINED_FOR_REFERENCE_VELOCITY = 1100
+    DEFINED_FOR_REFERENCE_VELOCITY = 800
+
     def get_mean_and_stddevs(self, sites, rup, dists, imt, stddev_types):
         """
         See :meth:`superclass method
