@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2014-2020 GEM Foundation
+# Copyright (C) 2014-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -241,6 +241,7 @@ validators = {
     'a_val': valid.floats,
     'bValue': valid.positivefloat,
     'b_val': valid.positivefloats,
+    'cornerMag': valid.positivefloat,
     'magScaleRel': valid.mag_scale_rel,
     'tectonicRegion': str,
     'ruptAspectRatio': valid.positivefloat,
@@ -259,6 +260,8 @@ validators = {
     'uncertaintyWeight': float,
     'alongStrike': valid.probability,
     'downDip': valid.probability,
+    'slipRate': valid.positivefloat,
+    'rigidity': valid.positivefloat,
     'totalMomentRate': valid.positivefloat,
     'characteristicRate': valid.positivefloat,
     'char_rate': valid.positivefloats,
@@ -408,7 +411,9 @@ def get(xml, investigation_time=50., rupture_mesh_spacing=5.,
         rupture_mesh_spacing,
         width_of_mfd_bin=width_of_mfd_bin,
         area_source_discretization=area_source_discretization)
-    return conv.convert_node(node)
+    src = conv.convert_node(node)
+    src.id = 0
+    return src
 
 
 if __name__ == '__main__':
