@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2015-2020 GEM Foundation
+# Copyright (C) 2015-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -15,6 +15,7 @@
 #
 # You should have received a copy of the GNU Affero General Public License
 # along with OpenQuake. If not, see <http://www.gnu.org/licenses/>.
+import os
 import operator
 import collections
 import pickle
@@ -24,17 +25,20 @@ import numpy
 
 from openquake.baselib import hdf5
 from openquake.baselib.general import groupby, block_splitter
-from openquake.baselib.node import context, striptag, Node
-from openquake.hazardlib import geo, mfd, pmf, source, tom
-from openquake.hazardlib import valid, InvalidFile
+from openquake.baselib.node import context, striptag, Node, node_to_dict
+from openquake.hazardlib import geo, mfd, pmf, source, tom, valid, InvalidFile
 from openquake.hazardlib.tom import PoissonTOM
 from openquake.hazardlib.source import NonParametricSeismicSource
 
 U32 = numpy.uint32
 F32 = numpy.float32
+F64 = numpy.float64
 EPSILON = 1E-12
-source_dt = numpy.dtype([('srcidx', U32), ('num_ruptures', U32),
+source_dt = numpy.dtype([('source_id', U32), ('num_ruptures', U32),
                          ('pik', hdf5.vuint8)])
+KNOWN_MFDS = ('incrementalMFD', 'truncGutenbergRichterMFD',
+              'arbitraryMFD', 'YoungsCoppersmithMFD', 'multiMFD',
+              'taperedGutenbergRichterMFD')
 
 
 def extract_dupl(values):
@@ -55,7 +59,7 @@ def fix_dupl(dist, fname=None, lineno=None):
     :param fname:
         the file which is being read; if it is None, it means you are writing
         the distribution: in that case raise an error for duplicated values
-    param lineno:
+    :param lineno:
         the line number of the file which is being read (None in writing mode)
     """
     n = len(dist)
@@ -64,6 +68,8 @@ def fix_dupl(dist, fname=None, lineno=None):
     # (strike, dip, rake) for a nodal plane distribution
     got = []
     for prob, value in dist:
+        if prob == 0:
+            raise ValueError('Zero probability in subnode %s' % value)
         values[value] += prob
         got.append(value)
     if len(values) < n:
@@ -211,10 +217,10 @@ class SourceGroup(collections.abc.Sequence):
         """
         assert src.tectonic_region_type == self.trt, (
             src.tectonic_region_type, self.trt)
-        if not src.min_mag:  # if not set already
-            src.min_mag = self.min_mag.get(self.trt) or self.min_mag['default']
-            if not src.get_mags():  # filtered out
-                return
+        src.min_mag = max(src.min_mag, self.min_mag.get(self.trt)
+                          or self.min_mag['default'])
+        if src.min_mag and not src.get_mags():  # filtered out
+            return
         # checking mutex ruptures
         if (not isinstance(src, NonParametricSeismicSource) and
                 self.rup_interdep == 'mutex'):
@@ -228,6 +234,16 @@ class SourceGroup(collections.abc.Sequence):
         if prev_max_mag is None or max_mag > prev_max_mag:
             self.max_mag = max_mag
 
+    def count_ruptures(self):
+        """
+        Set src.num_ruptures on each source in the group
+        """
+        for src in self:
+            src.nsites = 1
+            src.num_ruptures = src.count_ruptures()
+            print(src.weight)
+        return self
+
     def split(self, maxweight):
         """
         Split the group in subgroups with weight <= maxweight, unless it
@@ -325,6 +341,54 @@ def split_coords_3d(seq):
     return list(zip(lons, lats, depths))
 
 
+def convert_nonParametricSeismicSource(fname, node, rup_spacing=5.0):
+    """
+    Convert the given node into a non parametric source object.
+
+    :param fname:
+        full pathname to the XML file associated to the node
+    :param node:
+        a Node object coming from an XML file
+    :param rup_spacing:
+        Rupture spacing [km]
+    :returns:
+        a :class:`openquake.hazardlib.source.NonParametricSeismicSource`
+        instance
+    """
+    trt = node.attrib.get('tectonicRegion')
+    rups_weights = None
+    if 'rup_weights' in node.attrib:
+        rups_weights = F64(node['rup_weights'].split())
+    nps = source.NonParametricSeismicSource(
+        node['id'], node['name'], trt, [], [])
+    nps.splittable = 'rup_weights' not in node.attrib
+    path = os.path.splitext(fname)[0] + '.hdf5'
+    hdf5_fname = path if os.path.exists(path) else None
+    if hdf5_fname and node.text is None:
+        # gridded source, read the rupture data from the HDF5 file
+        with hdf5.File(hdf5_fname, 'r') as h:
+            dic = {k: d[:] for k, d in h[node['id']].items()}
+            nps.fromdict(dic, rups_weights)
+    else:
+        # read the rupture data from the XML nodes
+        num_probs = None
+        for i, rupnode in enumerate(node):
+            po = rupnode['probs_occur']
+            probs = pmf.PMF(valid.pmf(po))
+            if num_probs is None:  # first time
+                num_probs = len(probs.data)
+            elif len(probs.data) != num_probs:
+                # probs_occur must have uniform length for all ruptures
+                raise ValueError(
+                    'prob_occurs=%s has %d elements, expected %s'
+                    % (po, len(probs.data), num_probs))
+            rup = RuptureConverter(rup_spacing).convert_node(rupnode)
+            rup.tectonic_region_type = trt
+            rup.weight = None if rups_weights is None else rups_weights[i]
+            nps.data.append((rup, probs))
+    return nps
+
+
 class RuptureConverter(object):
     """
     Convert ruptures from nodes into Hazardlib ruptures.
@@ -411,6 +475,11 @@ class RuptureConverter(object):
            :class:`openquake.hazardlib.geo.GriddedSurface` instance
         4. there is a list of PlanarSurface nodes; returns a
            :class:`openquake.hazardlib.geo.MultiSurface` instance
+        5. there is either a single a kiteSurface or a list of kiteSurface
+           nodes; returns a
+           :class:`openquake.hazardlib.geo.MultiSurface` instance
+           or a :class:`openquake.hazardlib.geo.MultiSurface` instance,
+           respectively
 
         :param surface_nodes: surface nodes as just described
         """
@@ -431,6 +500,22 @@ class RuptureConverter(object):
                 coords = split_coords_3d(~surface_node.posList)
             points = [geo.Point(*p) for p in coords]
             surface = geo.GriddedSurface.from_points_list(points)
+        elif surface_node.tag.endswith('kiteSurface'):
+            # single or multiple kite surfaces
+            profs = []
+            for surface_node in surface_nodes:
+                profs.append(self.geo_lines(surface_node))
+            if len(profs) < 2:
+                surface = geo.KiteSurface.from_profiles(
+                    profs[0], self.rupture_mesh_spacing,
+                    self.rupture_mesh_spacing)
+            else:
+                surfaces = []
+                for prof in profs:
+                    surfaces.append(geo.KiteSurface.from_profiles(
+                        prof, self.rupture_mesh_spacing,
+                        self.rupture_mesh_spacing))
+                surface = geo.MultiSurface(surfaces)
         else:  # a collection of planar surfaces
             planar_surfaces = list(map(self.geo_planar, surface_nodes))
             surface = geo.MultiSurface(planar_surfaces)
@@ -482,6 +567,7 @@ class RuptureConverter(object):
             surface=self.convert_surfaces(surfaces))
         return rupt
 
+    # used in scenario only (?)
     def convert_multiPlanesRupture(self, node):
         """
         Convert a multiPlanesRupture node.
@@ -490,7 +576,17 @@ class RuptureConverter(object):
         """
         mag, rake, hypocenter = self.get_mag_rake_hypo(node)
         with context(self.fname, node):
-            surfaces = list(node.getnodes('planarSurface'))
+            if hasattr(node, 'planarSurface'):
+                surfaces = list(node.getnodes('planarSurface'))
+                for s in surfaces:
+                    assert s.tag.endswith('planarSurface')
+            elif hasattr(node, 'kiteSurface'):
+                surfaces = list(node.getnodes('kiteSurface'))
+                for s in surfaces:
+                    assert s.tag.endswith('kiteSurface')
+            else:
+                raise ValueError('Only multiSurfaces of planarSurfaces or'
+                                 'kiteSurfaces are supported (no mix)')
         rupt = source.rupture.BaseRupture(
             mag=mag, rake=rake,
             tectonic_region_type=None,
@@ -517,12 +613,12 @@ class RuptureConverter(object):
     def convert_ruptureCollection(self, node):
         """
         :param node: a ruptureCollection node
-        :returns: a dictionary grp_id -> EBRuptures
+        :returns: a dictionary et_id -> EBRuptures
         """
         coll = {}
         for grpnode in node:
-            grp_id = int(grpnode['id'])
-            coll[grp_id] = ebrs = []
+            et_id = int(grpnode['id'])
+            coll[et_id] = ebrs = []
             for node in grpnode:
                 rup = self.convert_node(node)
                 rup.rup_id = int(node['id'])
@@ -544,8 +640,7 @@ class SourceConverter(RuptureConverter):
                  complex_fault_mesh_spacing=None, width_of_mfd_bin=1.0,
                  area_source_discretization=None,
                  minimum_magnitude={'default': 0},
-                 spinning_floating=True, source_id=None,
-                 discard_trts=''):
+                 source_id=None, discard_trts=''):
         self.investigation_time = investigation_time
         self.area_source_discretization = area_source_discretization
         self.minimum_magnitude = minimum_magnitude
@@ -553,7 +648,6 @@ class SourceConverter(RuptureConverter):
         self.complex_fault_mesh_spacing = (
             complex_fault_mesh_spacing or rupture_mesh_spacing)
         self.width_of_mfd_bin = width_of_mfd_bin
-        self.spinning_floating = spinning_floating
         self.source_id = source_id
         self.discard_trts = discard_trts
 
@@ -571,6 +665,12 @@ class SourceConverter(RuptureConverter):
         source_id = getattr(obj, 'source_id', '')
         if self.source_id and source_id and source_id not in self.source_id:
             return
+        if hasattr(obj, 'mfd') and hasattr(obj.mfd, 'slip_rate'):
+            # TruncatedGRMFD with slip rate
+            m = obj.mfd
+            obj.mfd = m.from_slip_rate(
+                m.min_mag, m.max_mag, m.bin_width, m.b_val,
+                m.slip_rate, m.rigidity, obj.get_fault_surface_area())
         return obj
 
     def get_tom(self, node):
@@ -599,19 +699,27 @@ class SourceConverter(RuptureConverter):
         """
         with context(self.fname, node):
             [mfd_node] = [subnode for subnode in node
-                          if subnode.tag.endswith(
-                              ('incrementalMFD', 'truncGutenbergRichterMFD',
-                               'arbitraryMFD', 'YoungsCoppersmithMFD',
-                               'multiMFD'))]
+                          if subnode.tag.endswith(KNOWN_MFDS)]
             if mfd_node.tag.endswith('incrementalMFD'):
                 return mfd.EvenlyDiscretizedMFD(
                     min_mag=mfd_node['minMag'], bin_width=mfd_node['binWidth'],
                     occurrence_rates=~mfd_node.occurRates)
             elif mfd_node.tag.endswith('truncGutenbergRichterMFD'):
-                return mfd.TruncatedGRMFD(
-                    a_val=mfd_node['aValue'], b_val=mfd_node['bValue'],
-                    min_mag=mfd_node['minMag'], max_mag=mfd_node['maxMag'],
-                    bin_width=self.width_of_mfd_bin)
+                slip_rate = mfd_node.get('slipRate')
+                rigidity = mfd_node.get('rigidity')
+                if slip_rate:
+                    assert rigidity
+                    # instantiate with a NaN area, to be fixed later on
+                    gr_mfd = mfd.TruncatedGRMFD.from_slip_rate(
+                        mfd_node['minMag'], mfd_node['maxMag'],
+                        self.width_of_mfd_bin, mfd_node['bValue'],
+                        slip_rate, rigidity, area=numpy.nan)
+                else:
+                    gr_mfd = mfd.TruncatedGRMFD(
+                        a_val=mfd_node['aValue'], b_val=mfd_node['bValue'],
+                        min_mag=mfd_node['minMag'], max_mag=mfd_node['maxMag'],
+                        bin_width=self.width_of_mfd_bin)
+                return gr_mfd
             elif mfd_node.tag.endswith('arbitraryMFD'):
                 return mfd.ArbitraryMFD(
                     magnitudes=~mfd_node.magnitudes,
@@ -627,6 +735,11 @@ class SourceConverter(RuptureConverter):
             elif mfd_node.tag.endswith('multiMFD'):
                 return mfd.multi_mfd.MultiMFD.from_node(
                     mfd_node, self.width_of_mfd_bin)
+            elif mfd_node.tag.endswith('taperedGutenbergRichterMFD'):
+                return mfd.TaperedGRMFD(
+                    mfd_node['minMag'], mfd_node['maxMag'],
+                    mfd_node['cornerMag'], self.width_of_mfd_bin,
+                    mfd_node['aValue'], mfd_node['bValue'])
 
     def convert_npdist(self, node):
         """
@@ -644,11 +757,9 @@ class SourceConverter(RuptureConverter):
                 npdist.append((prob, geo.NodalPlane(strike, dip, rake)))
         with context(self.fname, npnode):
             fix_dupl(npdist, self.fname, npnode.lineno)
-            if not self.spinning_floating:
-                npdist = [(1, npdist[0][1])]  # consider the first nodal plane
             return pmf.PMF(npdist)
 
-    def convert_hpdist(self, node):
+    def convert_hddist(self, node):
         """
         Convert the given node into a probability mass function for the
         hypo depth distribution.
@@ -661,8 +772,6 @@ class SourceConverter(RuptureConverter):
             hddist = [(hd['probability'], hd['depth']) for hd in hdnode]
         with context(self.fname, hdnode):
             fix_dupl(hddist, self.fname, hdnode.lineno)
-            if not self.spinning_floating:  # consider the first hypocenter
-                hddist = [(1, hddist[0][1])]
             return pmf.PMF(hddist)
 
     def convert_areaSource(self, node):
@@ -694,7 +803,7 @@ class SourceConverter(RuptureConverter):
             upper_seismogenic_depth=~geom.upperSeismoDepth,
             lower_seismogenic_depth=~geom.lowerSeismoDepth,
             nodal_plane_distribution=self.convert_npdist(node),
-            hypocenter_distribution=self.convert_hpdist(node),
+            hypocenter_distribution=self.convert_hddist(node),
             polygon=polygon,
             area_discretization=area_discretization,
             temporal_occurrence_model=self.get_tom(node))
@@ -721,7 +830,7 @@ class SourceConverter(RuptureConverter):
             lower_seismogenic_depth=~geom.lowerSeismoDepth,
             location=geo.Point(*lon_lat),
             nodal_plane_distribution=self.convert_npdist(node),
-            hypocenter_distribution=self.convert_hpdist(node),
+            hypocenter_distribution=self.convert_hddist(node),
             temporal_occurrence_model=self.get_tom(node))
 
     def convert_multiPointSource(self, node):
@@ -744,7 +853,7 @@ class SourceConverter(RuptureConverter):
             upper_seismogenic_depth=~geom.upperSeismoDepth,
             lower_seismogenic_depth=~geom.lowerSeismoDepth,
             nodal_plane_distribution=self.convert_npdist(node),
-            hypocenter_distribution=self.convert_hpdist(node),
+            hypocenter_distribution=self.convert_hddist(node),
             mesh=geo.Mesh(F32(lons), F32(lats)),
             temporal_occurrence_model=self.get_tom(node))
 
@@ -787,6 +896,62 @@ class SourceConverter(RuptureConverter):
                 slip_list=slip_list)
         return simple
 
+    def convert_kiteFaultSource(self, node):
+        """
+        Convert the given node into a kite fault object.
+
+        :param node: a node with tag kiteFaultSource
+        :returns: a :class:`openquake.hazardlib.source.KiteFaultSource`
+                  instance
+        """
+        as_kite = True
+        try:
+            geom = node.simpleFaultGeometry
+            fault_trace = self.geo_line(geom)
+            as_kite = False
+        except:
+            geom = node.kiteFaultGeometry
+            profiles = self.geo_lines(geom)
+
+        msr = valid.SCALEREL[~node.magScaleRel]()
+        mfd = self.convert_mfdist(node)
+
+        with context(self.fname, node):
+            if as_kite:
+                outsrc = source.KiteFaultSource(
+                    source_id=node['id'],
+                    name=node['name'],
+                    tectonic_region_type=node.attrib.get('tectonicRegion'),
+                    mfd=mfd,
+                    rupture_mesh_spacing=self.rupture_mesh_spacing,
+                    magnitude_scaling_relationship=msr,
+                    rupture_aspect_ratio=~node.ruptAspectRatio,
+                    temporal_occurrence_model=self.get_tom(node),
+                    profiles=profiles,
+                    floating_x_step=1,
+                    floating_y_step=1,
+                    rake=~node.rake,
+                    profiles_sampling=None
+                    )
+            else:
+                outsrc = source.KiteFaultSource.as_simple_fault(
+                    source_id=node['id'],
+                    name=node['name'],
+                    tectonic_region_type=node.attrib.get('tectonicRegion'),
+                    mfd=mfd,
+                    rupture_mesh_spacing=self.rupture_mesh_spacing,
+                    magnitude_scaling_relationship=msr,
+                    rupture_aspect_ratio=~node.ruptAspectRatio,
+                    upper_seismogenic_depth=~geom.upperSeismoDepth,
+                    lower_seismogenic_depth=~geom.lowerSeismoDepth,
+                    fault_trace=fault_trace,
+                    dip=~geom.dip,
+                    rake=~node.rake,
+                    temporal_occurrence_model=self.get_tom(node),
+                    floating_x_step=1,
+                    floating_y_step=1)
+        return outsrc
+
     def convert_complexFaultSource(self, node):
         """
         Convert the given node into a complex fault object.
@@ -843,22 +1008,8 @@ class SourceConverter(RuptureConverter):
             a :class:`openquake.hazardlib.source.NonParametricSeismicSource`
             instance
         """
-        trt = node.attrib.get('tectonicRegion')
-        rup_pmf_data = []
-        rups_weights = None
-        if 'rup_weights' in node.attrib:
-            tmp = node.attrib.get('rup_weights')
-            rups_weights = numpy.array([float(s) for s in tmp.split()])
-        for i, rupnode in enumerate(node):
-            probs = pmf.PMF(valid.pmf(rupnode['probs_occur']))
-            rup = RuptureConverter.convert_node(self, rupnode)
-            rup.tectonic_region_type = trt
-            rup.weight = None if rups_weights is None else rups_weights[i]
-            rup_pmf_data.append((rup, probs))
-        nps = source.NonParametricSeismicSource(
-            node['id'], node['name'], trt, rup_pmf_data)
-        nps.splittable = 'rup_weights' not in node.attrib
-        return nps
+        return convert_nonParametricSeismicSource(self.fname, node,
+                                                  self.rupture_mesh_spacing)
 
     def convert_sourceModel(self, node):
         return [self.convert_node(subnode) for subnode in node]
@@ -895,7 +1046,7 @@ class SourceConverter(RuptureConverter):
             assert 'tom' in node.attrib, msg
             if isinstance(tom, PoissonTOM):
                 assert hasattr(sg, 'occurrence_rate')
-        #
+
         for src_node in node:
             src = self.convert_node(src_node)
             if src is None:  # filtered out by source_id
@@ -929,12 +1080,204 @@ class SourceConverter(RuptureConverter):
             raise ValueError(msg)
         return sg
 
+
+Row = collections.namedtuple(
+    'Row', 'id name code tectonicregion mfd magscalerel ruptaspectratio '
+    'upperseismodepth lowerseismodepth nodalplanedist hypodepthdist '
+    'geom coords')
+
+
+NPRow = collections.namedtuple(  # used for nonParametric sources
+    'NPRow', 'id name code tectonicregion ruptures geom coords')
+
+
+def _planar(surface):
+    poly = []
+    tl = surface.topLeft
+    poly.append((tl['lon'], tl['lat'], tl['depth']))
+    tr = surface.topRight
+    poly.append((tr['lon'], tr['lat'], tr['depth']))
+    br = surface.bottomRight
+    poly.append((br['lon'], br['lat'], br['depth']))
+    bl = surface.bottomLeft
+    poly.append((bl['lon'], bl['lat'], bl['depth']))
+    poly.append((tl['lon'], tl['lat'], tl['depth']))  # close the polygon
+    return [poly]
+
+
+class RowConverter(SourceConverter):
+    """
+    Used in the command oq nrml_to_csv to convert source models into
+    Row objects.
+    """
+    def convert_node(self, node):
+        """
+        Convert the given source node into a Row object
+        """
+        trt = node.attrib.get('tectonicRegion')
+        if trt and trt in self.discard_trts:
+            return
+        return getattr(self, 'convert_' + striptag(node.tag))(node)
+
+    def convert_mfdist(self, node):
+        with context(self.fname, node):
+            [mfd_node] = [subnode for subnode in node
+                          if subnode.tag.endswith(KNOWN_MFDS)]
+        return str(node_to_dict(mfd_node))
+
+    def convert_npdist(self, node):
+        lst = []
+        for w, np in super().convert_npdist(node).data:
+            dic = {'weight': w, 'dip': np.dip, 'rake': np.rake,
+                   'strike': np.strike}
+            lst.append(dic)
+        return str(lst)
+
+    def convert_hddist(self, node):
+        lst = []
+        for w, hd in super().convert_hddist(node).data:
+            lst.append(dict(weight=w, hypodepth=hd))
+        return str(lst)
+
+    def convert_areaSource(self, node):
+        geom = node.areaGeometry
+        coords = split_coords_2d(~geom.Polygon.exterior.LinearRing.posList)
+        coords += [coords[0]]  # close the polygon
+        # TODO: area_discretization = geom.attrib.get('discretization')
+        return Row(
+            node['id'],
+            node['name'],
+            'A',
+            node['tectonicRegion'],
+            self.convert_mfdist(node),
+            ~node.magScaleRel,
+            ~node.ruptAspectRatio,
+            ~geom.upperSeismoDepth,
+            ~geom.lowerSeismoDepth,
+            self.convert_npdist(node),
+            self.convert_hddist(node),
+            'Polygon', [coords])
+
+    def convert_pointSource(self, node):
+        geom = node.pointGeometry
+        return Row(
+            node['id'],
+            node['name'],
+            'P',
+            node['tectonicRegion'],
+            self.convert_mfdist(node),
+            ~node.magScaleRel,
+            ~node.ruptAspectRatio,
+            ~geom.upperSeismoDepth,
+            ~geom.lowerSeismoDepth,
+            self.convert_npdist(node),
+            self.convert_hddist(node),
+            'Point', ~geom.Point.pos)
+
+    def convert_multiPointSource(self, node):
+        geom = node.multiPointGeometry
+        coords = split_coords_2d(~geom.posList)
+        return Row(
+            node['id'],
+            node['name'],
+            'M',
+            node['tectonicRegion'],
+            self.convert_mfdist(node),
+            ~node.magScaleRel,
+            ~node.ruptAspectRatio,
+            ~geom.upperSeismoDepth,
+            ~geom.lowerSeismoDepth,
+            self.convert_npdist(node),
+            self.convert_hddist(node),
+            'MultiPoint', coords)
+
+    def convert_simpleFaultSource(self, node):
+        geom = node.simpleFaultGeometry
+        return Row(
+            node['id'],
+            node['name'],
+            'S',
+            node['tectonicRegion'],
+            self.convert_mfdist(node),
+            ~node.magScaleRel,
+            ~node.ruptAspectRatio,
+            ~geom.upperSeismoDepth,
+            ~geom.lowerSeismoDepth,
+            [{'dip': ~geom.dip, 'rake': ~node.rake}],
+            [],
+            'LineString', [(p.x, p.y) for p in self.geo_line(geom)])
+
+    def convert_complexFaultSource(self, node):
+        geom = node.complexFaultGeometry  # 1005
+        edges = []
+        for line in self.geo_lines(geom):
+            edges.append([(p.x, p.y, p.z) for p in line])
+        return Row(
+            node['id'],
+            node['name'],
+            'C',
+            node['tectonicRegion'],
+            self.convert_mfdist(node),
+            ~node.magScaleRel,
+            ~node.ruptAspectRatio,
+            numpy.nan,
+            numpy.nan,
+            [{'rake': ~node.rake}],
+            [],
+            '3D MultiLineString', edges)
+
+    def convert_characteristicFaultSource(self, node):
+        _, kind = node.surface[0].tag.split('}')
+        if kind == 'simpleFaultGeometry':
+            geom = 'LineString'
+            coords = [(point.x, point.y) for point in self.geo_line(
+                node.surface.simpleFaultGeometry)]
+        elif kind == 'complexFaultGeometry':
+            geom = '3D MultiLineString'
+            coords = []
+            for line in self.geo_lines(node.surface.complexFaultGeometry):
+                coords.append([(p.x, p.y, p.z) for p in line])
+        elif kind == 'planarSurface':
+            geom = '3D MultiPolygon'
+            coords = [_planar(surface) for surface in node.surface]
+        return Row(
+            node['id'],
+            node['name'],
+            'X',
+            node['tectonicRegion'],
+            self.convert_mfdist(node),
+            numpy.nan,
+            numpy.nan,
+            numpy.nan,
+            numpy.nan,
+            [{'rake': ~node.rake}],
+            [],
+            geom, coords)
+
+    def convert_nonParametricSeismicSource(self, node):
+        nps = convert_nonParametricSeismicSource(self.fname, node)
+        ruptures = []
+        for rup, pmf_ in nps.data:
+            probs = [p for p, i in pmf_.data]
+            hypo = [rup.hypocenter.x, rup.hypocenter.y, rup.hypocenter.z]
+            dic = dict(mag=rup.mag, rake=rup.rake, weight=rup.weight,
+                       probs_occur=probs, hypo=hypo)
+            ruptures.append(dic)
+        return NPRow(
+            node['id'],
+            node['name'],
+            'N',
+            node['tectonicRegion'],
+            ruptures,
+            'Polygon', [nps.polygon.coords])
+
+
 # ################### MultiPointSource conversion ######################## #
 
 
 def dists(node):
     """
-    :returns: hpdist, npdist and magScaleRel from the given pointSource node
+    :returns: hddist, npdist and magScaleRel from the given pointSource node
     """
     hd = tuple((node['probability'], node['depth'])
                for node in node.hypoDepthDist)
@@ -983,7 +1326,7 @@ def mfds2multimfd(mfds):
 
 
 def _pointsources2multipoints(srcs, i):
-    # converts pointSources with the same hpdist, npdist and msr into a
+    # converts pointSources with the same hddist, npdist and msr into a
     # single multiPointSource.
     allsources = []
     for (hd, npd, msr), sources in groupby(srcs, dists).items():
@@ -1024,17 +1367,22 @@ def _pointsources2multipoints(srcs, i):
     return i, allsources
 
 
+def drop_trivial_weights(group):
+    ws = group.attrib.get('srcs_weights')
+    if ws and len(set(ws)) == 1:  # all equal
+        del group.attrib['srcs_weights']
+
+
 def update_source_model(sm_node, fname):
     """
     :param sm_node: a sourceModel Node object containing sourceGroups
     """
     i = 0
     for group in sm_node:
-        if 'srcs_weights' in group.attrib:
-            raise InvalidFile('srcs_weights must be removed in %s' % fname)
         if not group.tag.endswith('sourceGroup'):
             raise InvalidFile('wrong NRML, got %s instead of '
                               'sourceGroup in %s' % (group.tag, fname))
+        drop_trivial_weights(group)
         psrcs = []
         others = []
         for src in group:
