@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2013-2020 GEM Foundation
+# Copyright (C) 2013-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -23,8 +23,9 @@ Validation library for the engine, the desktop tools, and anything else
 import os
 import re
 import ast
-import logging
+import json
 import toml
+import logging
 import numpy
 
 from openquake.baselib.general import distinct
@@ -32,7 +33,7 @@ from openquake.baselib import hdf5
 from openquake.hazardlib import imt, scalerel, gsim, pmf, site
 from openquake.hazardlib.gsim.base import registry, gsim_aliases
 from openquake.hazardlib.calc import disagg
-from openquake.hazardlib.calc.filters import IntegrationDistance
+from openquake.hazardlib.calc.filters import MagDepDistance, floatdict  # needed
 
 PRECISION = pmf.PRECISION
 
@@ -65,6 +66,7 @@ class FromFile(object):
     DEFINED_FOR_INTENSITY_MEASURE_TYPES = set()
     REQUIRES_SITES_PARAMETERS = set()
     REQUIRES_DISTANCES = set()
+    DEFINED_FOR_REFERENCE_VELOCITY = None
     kwargs = {}
 
     def init(self):
@@ -96,6 +98,18 @@ def to_toml(uncertainty):
     return text
 
 
+def _fix_toml(v):
+    # horrible hack to remove a pickle error with
+    # TomlDecoder.get_empty_inline_table.<locals>.DynamicInlineTableDict
+    # using toml.loads(s, _dict=dict) would be the right way, but it does
+    # not work :-(
+    if isinstance(v, numpy.ndarray):
+        return list(v)
+    elif hasattr(v, 'items'):
+        return {k1: _fix_toml(v1) for k1, v1 in v.items()}
+    return v
+
+
 # more tests are in tests/valid_test.py
 def gsim(value, basedir=''):
     """
@@ -106,6 +120,7 @@ def gsim(value, basedir=''):
     """
     value = to_toml(value)  # convert to TOML
     [(gsim_name, kwargs)] = toml.loads(value).items()
+    kwargs = _fix_toml(kwargs)
     for k, v in kwargs.items():
         if k.endswith(('_file', '_table')):
             kwargs[k] = os.path.normpath(os.path.join(basedir, v))
@@ -221,31 +236,6 @@ class Choices(Choice):
 export_formats = Choices('', 'xml', 'geojson', 'txt', 'csv', 'npz')
 
 
-def hazard_id(value):
-    """
-    >>> hazard_id('')
-    ()
-    >>> hazard_id('-1')
-    (-1,)
-    >>> hazard_id('42')
-    (42,)
-    >>> hazard_id('42,3')
-    (42, 3)
-    >>> hazard_id('42,3,4')
-    (42, 3, 4)
-    >>> hazard_id('42:3')
-    Traceback (most recent call last):
-       ...
-    ValueError: Invalid hazard_id '42:3'
-    """
-    if not value:
-        return ()
-    try:
-        return tuple(map(int, value.split(',')))
-    except Exception:
-        raise ValueError('Invalid hazard_id %r' % value)
-
-
 class Regex(object):
     """
     Compare the value with the given regex
@@ -293,9 +283,10 @@ class SimpleId(object):
 
 
 MAX_ID_LENGTH = 75  # length required for some sources in US14 collapsed model
-ASSET_ID_LENGTH = 100
+ASSET_ID_LENGTH = 50  # length that makes Murray happy
 
 simple_id = SimpleId(MAX_ID_LENGTH)
+branch_id = SimpleId(MAX_ID_LENGTH, r'^[\w\:\#_\-\.]+$')
 asset_id = SimpleId(ASSET_ID_LENGTH)
 source_id = SimpleId(MAX_ID_LENGTH, r'^[\w\.\-_]+$')
 nice_string = SimpleId(  # nice for Windows, Linux, HDF5 and XML
@@ -508,6 +499,8 @@ def coordinates(value):
     ...
     ValueError: Found overlapping site #2,  0 0 -1
     """
+    if isinstance(value, list):  # assume list of lists/tuples
+        return [point(' '.join(map(str, v))) for v in value]
     if not value.strip():
         raise ValueError('Empty list of coordinates: %r' % value)
     points = []
@@ -537,7 +530,12 @@ def positiveint(value):
     :param value: input string
     :returns: positive integer
     """
-    i = int(not_empty(value))
+    val = str(value).lower()
+    if val == 'true':
+        return 1
+    elif val == 'false':
+        return 0
+    i = int(not_empty(val))
     if i < 0:
         raise ValueError('integer %d < 0' % i)
     return i
@@ -601,7 +599,7 @@ def boolean(value):
         ...
     ValueError: Not a boolean: t
     """
-    value = value.strip().lower()
+    value = str(value).strip().lower()
     try:
         return _BOOL_DICT[value]
     except KeyError:
@@ -861,109 +859,6 @@ def dictionary(value):
     return dic
 
 
-# used for the maximum distance parameter in the job.ini file
-def floatdict(value):
-    """
-    :param value:
-        input string corresponding to a literal Python number or dictionary
-    :returns:
-        a Python dictionary key -> number
-
-    >>> floatdict("200")
-    {'default': 200}
-
-    >>> text = "{'active shallow crust': 250., 'default': 200}"
-    >>> sorted(floatdict(text).items())
-    [('active shallow crust', 250.0), ('default', 200)]
-    """
-    value = ast.literal_eval(value)
-    if isinstance(value, (int, float, list)):
-        return {'default': value}
-    dic = {'default': max(value.values())}
-    dic.update(value)
-    return dic
-
-
-def maximum_distance(value):
-    """
-    :param value:
-        input string corresponding to a valid maximum distance
-    :returns:
-        a IntegrationDistance mapping
-    """
-    return IntegrationDistance(MagDist.new(value).max())
-
-
-def unique_sorted(items):
-    """
-    Check that the items are unique and sorted
-    """
-    if len(set(items)) < len(items):
-        raise ValueError('Found duplicates in %s' % items)
-    elif items != sorted(items):
-        raise ValueError('%s is not ordered' % items)
-    return items
-
-
-class MagDist(dict):
-    """
-    A dictionary trt -> [(mag, dist), ...]
-    """
-    @classmethod
-    def new(cls, value):
-        """
-        :param value: string to be converted
-        :returns: MagDist dictionary
-
-        >>> md = MagDist.new('50')
-        >>> md
-        {'default': [(1, 50), (10, 50)]}
-        >>> md.max()
-        {'default': 50}
-        >>> md.interp(dict(default=['5.0', '5.1', '5.2']))
-        {'default': {'5.0': 50.0, '5.1': 50.0, '5.2': 50.0}}
-        """
-        items_by_trt = floatdict(value.replace('?', '-1'))
-        self = cls()
-        for trt, items in items_by_trt.items():
-            if isinstance(items, list):
-                self[trt] = unique_sorted(items)
-                for mag, dist in self[trt]:
-                    magnitude(mag)  # check valid magnitude
-            else:  # assume scalar distance
-                assert items == -1 or items >= 0, items
-                self[trt] = [(1, items), (10, items)]
-        return self
-
-    def interp(self, mags_by_trt):
-        """
-        :param mags_by_trt: a dictionary trt -> magnitudes as strings
-        :returns: a dictionary trt->mag->dist
-        """
-        dic = {}
-        for trt, mags in mags_by_trt.items():
-            xs, ys = zip(*self[trt])
-            if len(mags) == 1:
-                ms = [numpy.float64(mags)]
-            else:
-                ms = numpy.float64(mags)
-            dists = numpy.interp(ms, xs, ys)
-            dic[trt] = dict(zip(mags, dists))
-        return dic
-
-    def max(self):
-        """
-        :returns: a dictionary trt -> maxdist
-        """
-        return {trt: self[trt][-1][1] for trt in self}
-
-    def suggested(self):
-        """
-        :returns: True if there is a * for any TRT
-        """
-        return any(self[trt][-1][1] == -1 for trt in self)
-
-
 # ########################### SOURCES/RUPTURES ############################# #
 
 def mag_scale_rel(value):
@@ -993,7 +888,8 @@ def pmf(value):
     [(0.157, 0), (0.843, 1)]
     """
     probs = probabilities(value)
-    if abs(1.-sum(map(float, value.split()))) > 1e-12:
+    if sum(probs) != 1:
+        # avoid https://github.com/gem/oq-engine/issues/5901
         raise ValueError('The probabilities %s do not sum up to 1!' % value)
     return [(p, i) for i, p in enumerate(probs)]
 
@@ -1282,14 +1178,18 @@ class ParamSet(hdf5.LiteralAttrs, metaclass=MetaParamSet):
         Check if a dictionary name->string can be converted into a dictionary
         name->value. If the name does not correspond to a known parameter,
         print a warning.
+
+        :returns: a dictionary of converted parameters
         """
+        out = {}
         for name, text in dic.items():
             try:
                 p = getattr(cls, name)
             except AttributeError:
                 logging.warning('Ignored unknown parameter %s', name)
             else:
-                p.validator(text)
+                out[name] = p.validator(text)
+        return out
 
     @classmethod
     def from_(cls, dic):
@@ -1346,6 +1246,14 @@ class ParamSet(hdf5.LiteralAttrs, metaclass=MetaParamSet):
                 doc = docstring.format(**vars(self))
                 raise ValueError(doc)
 
+    def json(self):
+        """
+        :returns: the parameters as a JSON string
+        """
+        dic = {k: _fix_toml(v)
+               for k, v in self.__dict__.items() if not k.startswith('_')}
+        return json.dumps(dic)
+
     def __iter__(self):
         for item in sorted(vars(self).items()):
             yield item
