@@ -4,7 +4,7 @@
 #
 # LICENSE
 #
-# Copyright (C) 2010-2020 GEM Foundation, G. Weatherill, M. Pagani,
+# Copyright (C) 2010-2021 GEM Foundation, G. Weatherill, M. Pagani,
 # D. Monelli.
 #
 # The Hazard Modeller's Toolkit is free software: you can redistribute
@@ -53,7 +53,7 @@ Bird et al. 2010) for calculating seismic moment rate from Geodetic Strain
 
 import numpy as np
 from math import fabs
-import yaml
+import toml
 from openquake.hmtk.strain.strain_utils import (
     moment_function, calculate_taper_function)
 
@@ -224,7 +224,7 @@ class Shift(object):
 
         :param str region_parameter_file:
             To overwrite the default Bird et al (2007) classifcations the
-            regionalisation can be defined in a separate Yaml file
+            regionalisation can be defined in a separate TOML file
         '''
         base_params = base_params or IPL_PARAMS
         self.strain = None
@@ -244,8 +244,7 @@ class Shift(object):
         # If a regionalisation parameter file is defined then read
         # regionalisation from there - otherwise use Bird regionalisation
         if region_parameter_file:
-            self.regionalisation = yaml.safe_load(
-                open(region_parameter_file, 'rt'))
+            self.regionalisation = toml.load(open(region_parameter_file, 'rt'))
         else:
             self.regionalisation = BIRD_GLOBAL_PARAMETERS
 
