@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2015-2020 GEM Foundation
+# Copyright (C) 2015-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -56,10 +56,13 @@ def get_output(crmodel, assets_by_taxo, haz, rlzi=None):
     :param rlzi: if given, a realization index
     :returns: an ArrayWrapper loss_type -> array of shape (A, ...)
     """
+    primary = crmodel.primary_imtls
+    alias = {imt: 'gmv_%d' % i for i, imt in enumerate(primary)}
     if hasattr(haz, 'array'):  # classical
         eids = []
-        data = [haz.array[crmodel.imtls(imt), 0] for imt in crmodel.imtls]
-    elif isinstance(haz, numpy.ndarray):
+        data = {f'gmv_{m}': haz.array[crmodel.imtls(imt), 0]
+                for m, imt in enumerate(primary)}
+    elif set(haz.columns) - {'sid', 'eid', 'rlz'}:  # regular case
         # NB: in GMF-based calculations the order in which
         # the gmfs are stored is random since it depends on
         # which hazard task ends first; here we reorder
@@ -69,19 +72,17 @@ def get_output(crmodel, assets_by_taxo, haz, rlzi=None):
         # sample method would receive the means in random
         # order and produce random results even if the
         # seed is set correctly; very tricky indeed! (MS)
-        haz.sort(order='eid')
-        eids = haz['eid']
-        data = haz['gmv']  # shape (E, M)
-    elif haz == 0:  # no hazard for this site (event based)
+        haz = haz.sort_values('eid')
+        eids = haz.eid.to_numpy()
+        data = haz
+    else:  # ZeroGetter for this site (event based)
         eids = numpy.arange(1)
-        data = []
-    else:
-        raise ValueError('Unexpected haz=%s' % haz)
+        data = {f'gmv_{m}': [0] for m, imt in enumerate(primary)}
     dic = dict(eids=eids, assets=assets_by_taxo.assets,
-               loss_types=crmodel.loss_types)
+               loss_types=crmodel.loss_types, haz=haz)
     if rlzi is not None:
         dic['rlzi'] = rlzi
-    for l, lt in enumerate(crmodel.loss_types):
+    for lt in crmodel.loss_types:
         ls = []
         for taxonomy, assets_ in assets_by_taxo.items():
             if len(assets_by_taxo.eps):
@@ -89,14 +90,12 @@ def get_output(crmodel, assets_by_taxo, haz, rlzi=None):
             else:  # no CoVs
                 epsilons = ()
             arrays = []
-            rmodels, weights = crmodel.get_rmodels_weights(taxonomy)
+            rmodels, weights = crmodel.get_rmodels_weights(lt, taxonomy)
             for rm in rmodels:
-                if len(data) == 0:
-                    dat = [0]
-                elif len(eids):  # gmfs
-                    dat = data[:, rm.imti[lt]]
-                else:  # hcurves
-                    dat = data[rm.imti[lt]]
+                imt = rm.imt_by_lt[lt]
+                dat = data[alias.get(imt, imt)]
+                if hasattr(dat, 'to_numpy'):
+                    dat = dat.to_numpy()
                 arrays.append(rm(lt, assets_, dat, eids, epsilons))
             res = arrays[0] if len(arrays) == 1 else numpy.average(
                 arrays, weights=weights, axis=0)
@@ -116,8 +115,7 @@ class RiskInput(object):
     :param assets_by_site:
         array of assets, one per site
     """
-    def __init__(self, sid, hazard_getter, assets):
-        self.sid = sid
+    def __init__(self, hazard_getter, assets):
         self.hazard_getter = hazard_getter
         self.assets = assets
         self.weight = len(assets)
@@ -126,36 +124,36 @@ class RiskInput(object):
             aids.append(asset['ordinal'])
         self.aids = numpy.array(aids, numpy.uint32)
 
-    def gen_outputs(self, cr_model, monitor, tempname=None, haz=None):
+    def gen_outputs(self, crmodel, monitor, tempname=None, haz=None):
         """
         Group the assets per taxonomy and compute the outputs by using the
         underlying riskmodels. Yield one output per realization.
 
-        :param cr_model: a CompositeRiskModel instance
+        :param crmodel: a CompositeRiskModel instance
         :param monitor: a monitor object used to measure the performance
         """
         self.monitor = monitor
         hazard_getter = self.hazard_getter
-        [sid] = hazard_getter.sids
         if haz is None:
-            haz = hazard_getter.get_hazard()
-        if isinstance(haz, dict):
-            items = haz.items()
-        else:  # list of length R
-            items = enumerate(haz)
+            with monitor('getting hazard', measuremem=False):
+                haz = hazard_getter.get_hazard()
         with monitor('computing risk', measuremem=False):
             # this approach is slow for event_based_risk since a lot of
             # small arrays are passed (one per realization) instead of
             # a long array with all realizations; ebrisk does the right
             # thing since it calls get_output directly
             assets_by_taxo = get_assets_by_taxo(self.assets, tempname)
-            for rlzi, haz_by_rlzi in items:
-                out = get_output(cr_model, assets_by_taxo, haz_by_rlzi, rlzi)
-                yield out
+            if hasattr(haz, 'groupby'):  # DataFrame
+                for (sid, rlz), df in haz.groupby(['sid', 'rlz']):
+                    yield get_output(crmodel, assets_by_taxo, df, rlz)
+            else:  # list of probability curves
+                for rlz, pc in enumerate(haz):
+                    yield get_output(crmodel, assets_by_taxo, pc, rlz)
 
     def __repr__(self):
+        [sid] = self.hazard_getter.sids
         return '<%s sid=%s, %d asset(s)>' % (
-            self.__class__.__name__, self.sid, len(self.aids))
+            self.__class__.__name__, sid, len(self.aids))
 
 
 # used in scenario_risk
