@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2014-2020 GEM Foundation
+# Copyright (C) 2014-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -28,8 +28,8 @@ urlpatterns = [
     url(r'^(\d+)/abort$', views.calc_abort),
     url(r'^(\d+)/datastore$', views.calc_datastore),
     url(r'^(\d+)/extract/([-/_\.\w]+)$', views.extract),
-    url(r'^(\d+)/oqparam$', views.calc_oqparam),
     url(r'^(\d+)/results$', views.calc_results),
+    url(r'^(\d+)/hmap_(\d+)_(\d+)$', views.hmap_png),
     url(r'^(\d+)/traceback$', views.calc_traceback),
     url(r'^(\d+)/log/size$', views.calc_log_size),
     url(r'^(\d+)/log/(\d*):(\d*)$', views.calc_log),
